@@ -71,7 +71,10 @@ class QuerySet(object):
         # If inheritance is allowed, only return instances and instances of
         # subclasses of the class being used
         if document._meta.get('allow_inheritance') is True:
-            self._initial_query = {"_cls": {"$in": self._document._subclasses}}
+            if len(self._document._subclasses) == 1:
+                self._initial_query = {"_cls": self._document._subclasses[0]}
+            else:
+                self._initial_query = {"_cls": {"$in": self._document._subclasses}}
             self._loaded_fields = QueryFieldList(always_include=['_cls'])
         self._cursor_obj = None
         self._limit = None
@@ -104,13 +107,17 @@ class QuerySet(object):
                 raise InvalidQueryError(msg)
             query &= q_obj
 
-        queryset = self.clone()
+        if read_preference is None:
+            queryset = self.clone()
+        else:
+            # Use the clone provided when setting read_preference
+            queryset = self.read_preference(read_preference)
+
         queryset._query_obj &= query
         queryset._mongo_query = None
         queryset._cursor_obj = None
-        if read_preference is not None:
-            queryset.read_preference(read_preference)
         queryset._class_check = class_check
+
         return queryset
 
     def __len__(self):
@@ -178,7 +185,6 @@ class QuerySet(object):
             try:
                 queryset._cursor_obj = queryset._cursor[key]
                 queryset._skip, queryset._limit = key.start, key.stop
-                queryset._limit
                 if key.start and key.stop:
                     queryset._limit = key.stop - key.start
             except IndexError, err:
@@ -342,7 +348,7 @@ class QuerySet(object):
         """
         Document = _import_class('Document')
 
-        if not write_concern:
+        if write_concern is None:
             write_concern = {}
 
         docs = doc_or_docs
@@ -403,7 +409,7 @@ class QuerySet(object):
             self._len = count
         return count
 
-    def delete(self, write_concern=None):
+    def delete(self, write_concern=None, _from_doc_delete=False):
         """Delete the documents matched by the query.
 
         :param write_concern: Extra keyword arguments are passed down which
@@ -412,20 +418,25 @@ class QuerySet(object):
             ``save(..., write_concern={w: 2, fsync: True}, ...)`` will
             wait until at least two servers have recorded the write and
             will force an fsync on the primary server.
+        :param _from_doc_delete: True when called from document delete therefore
+            signals will have been triggered so don't loop.
         """
         queryset = self.clone()
         doc = queryset._document
 
+        if write_concern is None:
+            write_concern = {}
+
+        # Handle deletes where skips or limits have been applied or
+        # there is an untriggered delete signal
         has_delete_signal = signals.signals_available and (
             signals.pre_delete.has_receivers_for(self._document) or
             signals.post_delete.has_receivers_for(self._document))
 
-        if not write_concern:
-            write_concern = {}
+        call_document_delete = (queryset._skip or queryset._limit or
+                                has_delete_signal) and not _from_doc_delete
 
-        # Handle deletes where skips or limits have been applied or has a
-        # delete signal
-        if queryset._skip or queryset._limit or has_delete_signal:
+        if call_document_delete:
             for doc in queryset:
                 doc.delete(write_concern=write_concern)
             return
@@ -479,7 +490,7 @@ class QuerySet(object):
         if not update and not upsert:
             raise OperationError("No update parameters, would remove data")
 
-        if not write_concern:
+        if write_concern is None:
             write_concern = {}
 
         queryset = self.clone()
@@ -1479,7 +1490,7 @@ class QuerySet(object):
 
     # Deprecated
     def ensure_index(self, **kwargs):
-        """Deprecated use :func:`~Document.ensure_index`"""
+        """Deprecated use :func:`Document.ensure_index`"""
         msg = ("Doc.objects()._ensure_index() is deprecated. "
                "Use Doc.ensure_index() instead.")
         warnings.warn(msg, DeprecationWarning)
